/*
 * @(#)IKAuthTool.cpp	1.00 10th June 2000
 * 
 * Functions that take care of user authorization and security.
 *
 * Modification Log:
 */

/**
 *
 * IKTool is a sample application using iKey, an user identity token from
 * Rainbow Technologies (http://www.rainbow.com).
 * Using this application you can personalize an iKey, and store data either
 * protected with a pin number or without protection, as you desire.
 * 
 * Several generic functions have been written for iKey, so that they
 * can be used with any type of application. 
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 10th June, 2000
 */


#include <stdio.h>
#include <string.h>
#include "IKAuthTool.hxx"
#include "IKMessageTool.hxx"
#include "Tokenmd5.h"

int VerifyIKeyMasterKey(PRNBTKN_CONTEXT pContext, unsigned long *plSysErr, char *sMsg)
{
	// Hardcoded. This is the factory setting.
	// This MUST be reset in production environment.
	const char *sMasterKey = "rainbow";
	unsigned char sHashKey[16];

	// Make a 64-bit number from the key
	MD5_Hash((unsigned char *)sMasterKey, strlen(sMasterKey), sHashKey);

	*plSysErr = RnbTkn_VerifyMasterKey(pContext, sHashKey);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}

int VerifyIKeyPIN(PRNBTKN_CONTEXT pContext, unsigned long lKey,
				  unsigned long *plSysErr, char *sMsg)
{
	*plSysErr = RnbTkn_VerifyPIN(pContext, lKey);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}

int SetGlobalIKeyAccess(PRNBTKN_CONTEXT pContext, 
						unsigned char cMaxPinRetry,
						unsigned char cCurrPinCounter,
						unsigned char cCreateAccess,
						unsigned char cDeleteAccess,
						unsigned long *plSysErr, char *sMsg)
{
	RNBTKN_ACCESSINFO stAccessInfo;

	// This function needs MKEY verification first.
	if(0 != VerifyIKeyMasterKey(pContext, plSysErr, sMsg))
	{
		return -1;
	}

	stAccessInfo.MaxPinRetries = cMaxPinRetry;
	stAccessInfo.CurPinCounter = cCurrPinCounter;
	stAccessInfo.CreateAccess  = cCreateAccess;
	stAccessInfo.DeleteAccess  = cDeleteAccess;

	*plSysErr = RnbTkn_SetAccessSettings(pContext, &stAccessInfo);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}

int ModifyIKeyPIN(PRNBTKN_CONTEXT pContext, 
				  unsigned long lOldKey, unsigned long lNewKey,
				  unsigned long *plSysErr, char *sMsg)
{
	// This function needs MKEY or PIN verification first.
	if(0 != VerifyIKeyMasterKey(pContext, plSysErr, sMsg))
	{
		return -1;
	}

	*plSysErr = RnbTkn_ModifyPIN(pContext, lNewKey);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}


// Random nember generation
//
// Call RnbTkn_GetChallenge to get a 64bit random number.

// Implementing data security with MD5 method.
// 
// Call RnbTkn_HashToken to get a MD5 digest.
// This can be used as a checksum to prevent data tamper.

// Implementing data security with HMAC-MD5 method.
//
// A method to create a checksum value for a data. 
//
// The key is a secret value known only to the message creator 
// and the message receiver. And the key is not stored in the iKey.
// A pair of generated keys are needed for this process.
//

// Steps to create the keys:
// 1. Decide on a secret value. Lets say "rainbow".
// 2. Call function HMAC_MD5 to create two keys and a digest.
//    Pass text and text_len as 0.
// 3. Write the two keys into the iKey application directory as
//    separate files.
// Note that the secret is not stored in the iKey.
// And the digest is not used.
//
int CreateIKeyHMACKeys(unsigned char *sKey1, unsigned char *sKey2, unsigned char *pSecret, int iSecretLen)
{
	unsigned char sDigest[16];
	HMAC_MD5(0, 0, pSecret, iSecretLen, sKey1, sKey2, sDigest);
	return 0;
}


// Steps to get checksum:
// 1. Call RnbTkn_HMAC_MD5 with the two key file ids and the text to
//    hash to get the digest.
//
// Steps to verify checksum:
// 1. Call HMAC_MD5 with the secret value.
//    Pass the text and text_len whose checksum is to be verified.
//    Pass key1 and key2 as 0.
// 2. Compare the digest returned with the checksum supplied by the sender.
//
int VerifyIKeyHMACKeys(PRNBTKN_CONTEXT pContext, 
					   unsigned long lDirId, 
					   unsigned long lKeyFile1, unsigned long lKeyFile2,
					   unsigned char *pSecret, int iSecretLen,
					   unsigned long *plSysErr, char *sMsg)
{
	// this is a sample text to encode. 
	// This may be constructed run time for additional security.
	static const char *sTextToEncode = "yourownkey";
	unsigned char sDigest1[16], sDigest2[16];

	*plSysErr = RnbTkn_HMAC_MD5(pContext, 
					lDirId, lKeyFile1, lDirId, lKeyFile2, 
					(unsigned char *)sTextToEncode, strlen(sTextToEncode),
					sDigest1);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}

	HMAC_MD5((unsigned char *)sTextToEncode, strlen(sTextToEncode), pSecret, iSecretLen, NULL, NULL, sDigest2);
	if(0 != memcmp(sDigest1, sDigest2, 16))
	{
		sprintf(sMsg, "Verification of HMAC checksum failed.");
		return -1;
	}
	return 0;
}
